var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/sdk.ts
var HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
var highlight_obj;
var H = {
  init: (options = {}) => {
    try {
      highlight_obj = new Highlight(options);
    } catch (e) {
      console.log("highlight-node init error: ", e);
    }
  },
  isInitialized: () => !!highlight_obj,
  consumeError: (error, secureSessionId, requestId) => {
    try {
      highlight_obj.consumeCustomError(error, secureSessionId, requestId);
    } catch (e) {
      console.log("highlight-node consumeError error: ", e);
    }
  },
  consumeEvent: (secureSessionId) => {
    try {
      highlight_obj.consumeCustomEvent(secureSessionId);
    } catch (e) {
      console.log("highlight-node consumeEvent error: ", e);
    }
  },
  recordMetric: (secureSessionId, name, value, requestId, tags) => {
    try {
      highlight_obj.recordMetric(
        secureSessionId,
        name,
        value,
        requestId,
        tags
      );
    } catch (e) {
      console.log("highlight-node recordMetric error: ", e);
    }
  },
  flush: async () => {
    try {
      await highlight_obj.flush();
    } catch (e) {
      console.log("highlight-node flush error: ", e);
    }
  }
};

// src/graph/generated/operations.ts
import gql from "graphql-tag";
var PushBackendPayloadDocument = gql`
	mutation PushBackendPayload($errors: [BackendErrorObjectInput]!) {
		pushBackendPayload(errors: $errors)
	}
`;
var MarkBackendSetupDocument = gql`
	mutation MarkBackendSetup($session_secure_id: String!) {
		markBackendSetup(session_secure_id: $session_secure_id)
	}
`;
var PushMetricsDocument = gql`
	mutation PushMetrics($metrics: [MetricInput]!) {
		pushMetrics(metrics: $metrics)
	}
`;
var defaultWrapper = (action, _operationName, _operationType) => action();
function getSdk(client, withWrapper = defaultWrapper) {
  return {
    PushBackendPayload(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          PushBackendPayloadDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "PushBackendPayload",
        "mutation"
      );
    },
    MarkBackendSetup(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          MarkBackendSetupDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "MarkBackendSetup",
        "mutation"
      );
    },
    PushMetrics(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          PushMetricsDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "PushMetrics",
        "mutation"
      );
    }
  };
}

// src/client.ts
import ErrorStackParser from "error-stack-parser";
import { GraphQLClient } from "graphql-request";

// src/errorContext.ts
import LRU from "lru-cache";
import fs from "fs";
var CONTEXT_LINE_MAX_SIZE = 1e3;
var CONTEXT_LINES_COUNT = 5;
function coalesceSize(inputSize) {
  if (inputSize === void 0) {
    return CONTEXT_CACHE_DEFAULT_SIZE;
  } else if (inputSize <= 0) {
    return void 0;
  } else {
    return inputSize;
  }
}
var CONTEXT_CACHE_DEFAULT_SIZE = 10;
var ErrorContext = class {
  constructor(options) {
    this._sourceContextCacheSizeMB = coalesceSize(
      options.sourceContextCacheSizeMB
    );
    const lruOptions = {
      max: 1e3,
      maxSize: this._sourceContextCacheSizeMB ? this._sourceContextCacheSizeMB * 1e3 * 1e3 : void 0,
      sizeCalculation: (lines, key) => {
        return lines.size;
      }
    };
    this._cache = new LRU(lruOptions);
  }
  _getOrLoadLines(filename) {
    const maybeCached = this._cache.get(filename);
    if (maybeCached !== void 0) {
      return maybeCached.lines;
    }
    let size = 0;
    const contents = fs.readFileSync(filename, "utf8");
    const lineStrs = contents.split("\n");
    const lines = /* @__PURE__ */ new Map();
    lineStrs.forEach((line, idx) => {
      const curLine = line.substring(0, CONTEXT_LINE_MAX_SIZE) + "\n";
      lines.set(idx + 1, curLine);
      size += curLine.length;
    });
    this._cache.set(filename, {
      lines,
      size
    });
    return lines;
  }
  getStackFrameContext(filename, lineNumber) {
    const lines = this._getOrLoadLines(filename);
    let lineContent = lines.get(lineNumber);
    let linesBefore = "";
    for (let i = lineNumber - CONTEXT_LINES_COUNT; i < lineNumber; i++) {
      let nextLine = lines.get(i);
      if (nextLine !== void 0) {
        linesBefore += nextLine;
      }
    }
    let linesAfter = "";
    for (let i = lineNumber + 1; i < lineNumber + CONTEXT_LINES_COUNT + 1; i++) {
      let nextLine = lines.get(i);
      if (nextLine !== void 0) {
        linesAfter += nextLine;
      }
    }
    return {
      lineContent,
      linesBefore,
      linesAfter
    };
  }
};

// src/client.ts
var Highlight = class {
  constructor(options) {
    this.FLUSH_TIMEOUT = 10;
    this.BACKEND_SETUP_TIMEOUT = 15 * 60 * 1e3;
    this.errors = [];
    this.metrics = [];
    this.lastBackendSetupEvent = 0;
    this._backendUrl = options.backendUrl || "https://pub.highlight.run";
    const client = new GraphQLClient(this._backendUrl, {
      headers: {}
    });
    this._graphqlSdk = getSdk(client);
    this._intervalFunction = setInterval(
      () => this.flush(),
      this.FLUSH_TIMEOUT * 1e3
    );
    if (!options.disableErrorSourceContext) {
      this._errorContext = new ErrorContext({
        sourceContextCacheSizeMB: options.errorSourceContextCacheSizeMB
      });
    }
  }
  recordMetric(secureSessionId, name, value, requestId, tags) {
    this.metrics.push({
      session_secure_id: secureSessionId,
      group: requestId,
      name,
      value,
      category: "BACKEND",
      timestamp: new Date().toISOString(),
      tags
    });
  }
  consumeCustomError(error, secureSessionId, requestId) {
    let res = [];
    try {
      res = ErrorStackParser.parse(error);
      res = res.map((frame) => {
        var _a;
        try {
          if (frame.fileName !== void 0 && frame.lineNumber !== void 0) {
            const context = (_a = this._errorContext) == null ? void 0 : _a.getStackFrameContext(
              frame.fileName,
              frame.lineNumber
            );
            return { ...frame, ...context };
          }
        } catch {
        }
        return frame;
      });
    } catch {
    }
    this.errors.push({
      event: error.message ? `${error.name}: ${error.message}` : `${error.name}`,
      request_id: requestId,
      session_secure_id: secureSessionId,
      source: "",
      stackTrace: JSON.stringify(res),
      timestamp: new Date().toISOString(),
      type: "BACKEND",
      url: ""
    });
  }
  consumeCustomEvent(secureSessionId) {
    const sendBackendSetup = Date.now() - this.lastBackendSetupEvent > this.BACKEND_SETUP_TIMEOUT;
    if (sendBackendSetup) {
      this._graphqlSdk.MarkBackendSetup({
        session_secure_id: secureSessionId
      }).then(() => {
        this.lastBackendSetupEvent = Date.now();
      }).catch((e) => {
        console.log("highlight-node error: ", e);
      });
    }
  }
  async flushErrors() {
    if (this.errors.length === 0) {
      return;
    }
    const variables = {
      errors: this.errors
    };
    this.errors = [];
    try {
      await this._graphqlSdk.PushBackendPayload(variables);
    } catch (e) {
      console.log("highlight-node pushErrors error: ", e);
    }
  }
  async flushMetrics() {
    if (this.metrics.length === 0) {
      return;
    }
    const variables = {
      metrics: this.metrics
    };
    this.metrics = [];
    try {
      await this._graphqlSdk.PushMetrics(variables);
    } catch (e) {
      console.log("highlight-node pushMetrics error: ", e);
    }
  }
  async flush() {
    await Promise.all([this.flushErrors(), this.flushMetrics()]);
  }
};

// src/handlers.ts
var handlers_exports = {};
__export(handlers_exports, {
  errorHandler: () => errorHandler
});
function errorHandler(options = {}) {
  return async (error, req, res, next) => {
    try {
      if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId, requestId] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        if (secureSessionId && requestId) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          H.consumeEvent(secureSessionId);
          if (error instanceof Error) {
            H.consumeError(error, secureSessionId, requestId);
          }
        }
      }
      next(error);
    } catch {
      next(error);
    }
  };
}
export {
  H,
  HIGHLIGHT_REQUEST_HEADER,
  handlers_exports as Handlers,
  Highlight
};
